#include "nemo-sync-prompt-dialog.h"
#include "libnemo-extension/nemo-extension-i18n.h"

static GtkWidget *dialog = NULL;
static GtkWidget *window = NULL;

static void
create_sync_prompt_dialog (const char *primary_text, 
                           const char *secondary_text, 
                           const char *sync_text)
{
	GtkWidget *content_area;
	GtkWidget *progress_bar;
	
	if (!window) {
	    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	}
	
	if (!dialog) {
	    dialog = gtk_message_dialog_new (GTK_WINDOW (window), 
	                                     GTK_DIALOG_DESTROY_WITH_PARENT, 
	                                     GTK_MESSAGE_INFO, 
	                                     GTK_BUTTONS_NONE, 
	                                     NULL);
    }
	
	g_object_set (dialog, 
	              "text", primary_text, 
	              "secondary-text", secondary_text, 
	              NULL);
	              
	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
	
	progress_bar = gtk_progress_bar_new ();
	gtk_progress_bar_set_show_text (GTK_PROGRESS_BAR (progress_bar), TRUE);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progress_bar), sync_text);

	gtk_box_pack_start (GTK_BOX (content_area), progress_bar, TRUE, TRUE, 0);
}

static void
update_progress_bar (GtkDialog *dialog, 
                     gdouble fraction)
{
    GtkWidget *content_area;
	GList *children = NULL;
	GList *child = NULL;
	GtkWidget *progress_bar;
	const char *done_text;
	
	content_area = gtk_dialog_get_content_area (dialog);
	children = gtk_container_get_children (GTK_CONTAINER (content_area));
	for (child = children; child != NULL; child = child->next) {
	    const gchar *child_type = g_type_name (G_TYPE_FROM_INSTANCE (child->data));
        if (g_strcmp0 (child_type, "GtkProgressBar") == 0) {
            progress_bar = GTK_WIDGET (child->data);
        }
	}
	
	done_text = _("Data synchronization is complete");
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progress_bar), done_text);
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), fraction);
}

static gboolean
on_timeout_callback (gpointer user_data)
{
    gtk_widget_destroy (window);
    window = NULL;
    dialog = NULL;
    
    return FALSE;
}

static void
sync_prompt_dialog_destroy (GtkDialog *dialog, 
                            gint response_id, 
                            gpointer   user_data)
{
	update_progress_bar (dialog, 1.0);
	
	//小文件要显示进度，延时销毁
	g_timeout_add (500, 
	               on_timeout_callback, 
	               NULL);
}

static void
show_sync_prompt_dialog ()
{
    const char *primary_text;
    const char *secondary_text;
    const char *sync_text;
    //gint result;
    
    primary_text = _("Writing to cache, please don't unplug the removable device, otherwise data will be lost.");
    secondary_text = NULL;
    sync_text = _("Syncing data to device...");
    create_sync_prompt_dialog (primary_text, secondary_text, sync_text);
    gtk_widget_show_all (dialog);
    
    g_signal_connect (dialog, 
                      "response", 
                      G_CALLBACK (sync_prompt_dialog_destroy), NULL);
    
    //result = gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_dialog_run (GTK_DIALOG (dialog));
    
    //gtk_widget_destroy (dialog);
}

static gboolean
idle_callback (gpointer user_data)
{
	show_sync_prompt_dialog ();
	
	return FALSE;
}

void
show_sync_prompt_dialog_idle ()
{
    //g_idle_add (idle_callback, NULL);
    g_timeout_add (0, idle_callback, NULL);
}

void
sync_prompt_dialog_sync_done ()
{
    if (dialog) {
        g_signal_emit_by_name (dialog, "response");
    }
}
